@@ -10,8 +10,8 @@ import sys
 import numpy as np
 import pandas
 import pandas.api.types
-import pandas.util.testing as tm
-from pandas import DataFrame, NaT
+import pandas.testing as tm
+from pandas import DataFrame
 
 try:
     import pkg_resources  # noqa
@@ -26,14 +26,24 @@ import pandas_gbq.schema
 
 TABLE_ID = "new_test"
 PANDAS_VERSION = pkg_resources.parse_version(pandas.__version__)
-NULLABLE_INT_PANDAS_VERSION = pkg_resources.parse_version("0.24.0")
-NULLABLE_INT_MESSAGE = "Require pandas 0.24+ in order to use nullable integer type."
 
 
 def test_imports():
     gbq._test_google_api_imports()
 
 
+def make_mixed_dataframe_v1():
+    # Re-implementation of private pandas.util.testing.makeMixedDataFrame
+    return pandas.DataFrame(
+        {
+            "A": [0.0, 1.0, 2.0, 3.0, 4.0],
+            "B": [0.0, 1.0, 0.0, 1.0, 0.0],
+            "C": ["foo1", "foo2", "foo3", "foo4", "foo5"],
+            "D": pandas.bdate_range("1/1/2009", periods=5),
+        }
+    )
+
+
 def make_mixed_dataframe_v2(test_size):
     # create df to test for all BQ datatypes except RECORD
     bools = np.random.randint(2, size=(1, test_size)).astype(bool)
@@ -142,328 +152,6 @@ class TestReadGBQIntegration(object):
         self.gbq_connector = gbq.GbqConnector(project, credentials=credentials)
         self.credentials = credentials
 
-    def test_should_properly_handle_empty_strings(self, project_id):
-        query = 'SELECT "" AS empty_string'
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        tm.assert_frame_equal(df, DataFrame({"empty_string": [""]}))
-
-    def test_should_properly_handle_null_strings(self, project_id):
-        query = "SELECT STRING(NULL) AS null_string"
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        tm.assert_frame_equal(df, DataFrame({"null_string": [None]}))
-
-    def test_should_properly_handle_valid_integers(self, project_id):
-        query = "SELECT CAST(3 AS INT64) AS valid_integer"
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="standard",
-        )
-        tm.assert_frame_equal(df, DataFrame({"valid_integer": [3]}))
-
-    def test_should_properly_handle_nullable_integers(self, project_id):
-        if PANDAS_VERSION < NULLABLE_INT_PANDAS_VERSION:
-            pytest.skip(msg=NULLABLE_INT_MESSAGE)
-
-        query = """SELECT * FROM
-                    UNNEST([1, NULL]) AS nullable_integer
-                """
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="standard",
-            dtypes={"nullable_integer": "Int64"},
-        )
-        tm.assert_frame_equal(
-            df,
-            DataFrame(
-                {"nullable_integer": pandas.Series([1, pandas.NA], dtype="Int64")}
-            ),
-        )
-
-    def test_should_properly_handle_valid_longs(self, project_id):
-        query = "SELECT 1 << 62 AS valid_long"
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="standard",
-        )
-        tm.assert_frame_equal(df, DataFrame({"valid_long": [1 << 62]}))
-
-    def test_should_properly_handle_nullable_longs(self, project_id):
-        if PANDAS_VERSION < NULLABLE_INT_PANDAS_VERSION:
-            pytest.skip(msg=NULLABLE_INT_MESSAGE)
-
-        query = """SELECT * FROM
-                    UNNEST([1 << 62, NULL]) AS nullable_long
-                """
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="standard",
-            dtypes={"nullable_long": "Int64"},
-        )
-        tm.assert_frame_equal(
-            df,
-            DataFrame(
-                {"nullable_long": pandas.Series([1 << 62, pandas.NA], dtype="Int64")}
-            ),
-        )
-
-    def test_should_properly_handle_null_integers(self, project_id):
-        if PANDAS_VERSION < NULLABLE_INT_PANDAS_VERSION:
-            pytest.skip(msg=NULLABLE_INT_MESSAGE)
-
-        query = "SELECT CAST(NULL AS INT64) AS null_integer"
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="standard",
-            dtypes={"null_integer": "Int64"},
-        )
-        tm.assert_frame_equal(
-            df, DataFrame({"null_integer": pandas.Series([pandas.NA], dtype="Int64")}),
-        )
-
-    def test_should_properly_handle_valid_floats(self, project_id):
-        from math import pi
-
-        query = "SELECT PI() AS valid_float"
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        tm.assert_frame_equal(df, DataFrame({"valid_float": [pi]}))
-
-    def test_should_properly_handle_nullable_floats(self, project_id):
-        from math import pi
-
-        query = """SELECT * FROM
-                    (SELECT PI() AS nullable_float),
-                    (SELECT NULL AS nullable_float)"""
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        tm.assert_frame_equal(df, DataFrame({"nullable_float": [pi, None]}))
-
-    def test_should_properly_handle_valid_doubles(self, project_id):
-        from math import pi
-
-        query = "SELECT PI() * POW(10, 307) AS valid_double"
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        tm.assert_frame_equal(df, DataFrame({"valid_double": [pi * 10 ** 307]}))
-
-    def test_should_properly_handle_nullable_doubles(self, project_id):
-        from math import pi
-
-        query = """SELECT * FROM
-                    (SELECT PI() * POW(10, 307) AS nullable_double),
-                    (SELECT NULL AS nullable_double)"""
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        tm.assert_frame_equal(
-            df, DataFrame({"nullable_double": [pi * 10 ** 307, None]})
-        )
-
-    def test_should_properly_handle_null_floats(self, project_id):
-        query = """SELECT null_float
-        FROM UNNEST(ARRAY<FLOAT64>[NULL, 1.0]) AS null_float
-        """
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="standard",
-        )
-        tm.assert_frame_equal(df, DataFrame({"null_float": [np.nan, 1.0]}))
-
-    def test_should_properly_handle_date(self, project_id):
-        query = "SELECT DATE(2003, 1, 4) AS date_col"
-        df = gbq.read_gbq(query, project_id=project_id, credentials=self.credentials,)
-        expected = DataFrame(
-            {
-                "date_col": pandas.Series(
-                    [datetime.date(2003, 1, 4)], dtype="datetime64[ns]"
-                )
-            },
-        )
-        tm.assert_frame_equal(df, expected)
-
-    def test_should_properly_handle_time(self, project_id):
-        query = (
-            "SELECT TIME_ADD(TIME(3, 14, 15), INTERVAL 926589 MICROSECOND) AS time_col"
-        )
-        df = gbq.read_gbq(query, project_id=project_id, credentials=self.credentials,)
-        expected = DataFrame(
-            {
-                "time_col": pandas.Series(
-                    [datetime.time(3, 14, 15, 926589)], dtype="object"
-                )
-            },
-        )
-        tm.assert_frame_equal(df, expected)
-
-    def test_should_properly_handle_timestamp_unix_epoch(self, project_id):
-        query = 'SELECT TIMESTAMP("1970-01-01 00:00:00") AS unix_epoch'
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        expected = DataFrame(
-            {"unix_epoch": ["1970-01-01T00:00:00.000000Z"]}, dtype="datetime64[ns]",
-        )
-        if expected["unix_epoch"].dt.tz is None:
-            expected["unix_epoch"] = expected["unix_epoch"].dt.tz_localize("UTC")
-        tm.assert_frame_equal(df, expected)
-
-    def test_should_properly_handle_arbitrary_timestamp(self, project_id):
-        query = 'SELECT TIMESTAMP("2004-09-15 05:00:00") AS valid_timestamp'
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        expected = DataFrame(
-            {"valid_timestamp": ["2004-09-15T05:00:00.000000Z"]},
-            dtype="datetime64[ns]",
-        )
-        if expected["valid_timestamp"].dt.tz is None:
-            expected["valid_timestamp"] = expected["valid_timestamp"].dt.tz_localize(
-                "UTC"
-            )
-        tm.assert_frame_equal(df, expected)
-
-    def test_should_properly_handle_datetime_unix_epoch(self, project_id):
-        query = 'SELECT DATETIME("1970-01-01 00:00:00") AS unix_epoch'
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        tm.assert_frame_equal(
-            df,
-            DataFrame({"unix_epoch": ["1970-01-01T00:00:00"]}, dtype="datetime64[ns]"),
-        )
-
-    def test_should_properly_handle_arbitrary_datetime(self, project_id):
-        query = 'SELECT DATETIME("2004-09-15 05:00:00") AS valid_timestamp'
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        tm.assert_frame_equal(
-            df, DataFrame({"valid_timestamp": [np.datetime64("2004-09-15T05:00:00")]}),
-        )
-
-    @pytest.mark.parametrize(
-        "expression, is_expected_dtype",
-        [
-            ("current_date()", pandas.api.types.is_datetime64_ns_dtype),
-            ("current_timestamp()", pandas.api.types.is_datetime64tz_dtype),
-            ("current_datetime()", pandas.api.types.is_datetime64_ns_dtype),
-            ("TRUE", pandas.api.types.is_bool_dtype),
-            ("FALSE", pandas.api.types.is_bool_dtype),
-        ],
-    )
-    def test_return_correct_types(self, project_id, expression, is_expected_dtype):
-        """
-        All type checks can be added to this function using additional
-        parameters, rather than creating additional functions.
-        We can consolidate the existing functions here in time
-
-        TODO: time doesn't currently parse
-        ("time(12,30,00)", "<M8[ns]"),
-        """
-        query = "SELECT {} AS _".format(expression)
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="standard",
-        )
-        assert is_expected_dtype(df["_"].dtype)
-
-    def test_should_properly_handle_null_timestamp(self, project_id):
-        query = "SELECT TIMESTAMP(NULL) AS null_timestamp"
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        expected = DataFrame({"null_timestamp": [NaT]}, dtype="datetime64[ns]")
-        expected["null_timestamp"] = expected["null_timestamp"].dt.tz_localize("UTC")
-        tm.assert_frame_equal(df, expected)
-
-    def test_should_properly_handle_null_datetime(self, project_id):
-        query = "SELECT CAST(NULL AS DATETIME) AS null_datetime"
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="standard",
-        )
-        tm.assert_frame_equal(df, DataFrame({"null_datetime": [NaT]}))
-
-    def test_should_properly_handle_null_boolean(self, project_id):
-        query = "SELECT BOOLEAN(NULL) AS null_boolean"
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        tm.assert_frame_equal(df, DataFrame({"null_boolean": [None]}))
-
-    def test_should_properly_handle_nullable_booleans(self, project_id):
-        query = """SELECT * FROM
-                    (SELECT BOOLEAN(TRUE) AS nullable_boolean),
-                    (SELECT NULL AS nullable_boolean)"""
-        df = gbq.read_gbq(
-            query,
-            project_id=project_id,
-            credentials=self.credentials,
-            dialect="legacy",
-        )
-        tm.assert_frame_equal(
-            df, DataFrame({"nullable_boolean": [True, None]}).astype(object)
-        )
-
     def test_unicode_string_conversion_and_normalization(self, project_id):
         correct_test_datatype = DataFrame({"unicode_string": ["éü"]})
         unicode_string = "éü"
@@ -617,26 +305,6 @@ class TestReadGBQIntegration(object):
         )
         assert len(df) == 10
 
-    def test_zero_rows(self, project_id):
-        # Bug fix for https://github.com/pandas-dev/pandas/issues/10273
-        df = gbq.read_gbq(
-            'SELECT name, number, (mlc_class = "HU") is_hurricane, iso_time '
-            "FROM `bigquery-public-data.noaa_hurricanes.hurricanes` "
-            'WHERE iso_time = TIMESTAMP("1900-01-01 00:00:00") ',
-            project_id=project_id,
-            credentials=self.credentials,
-        )
-        empty_columns = {
-            "name": pandas.Series([], dtype=object),
-            "number": pandas.Series([], dtype=np.dtype(int)),
-            "is_hurricane": pandas.Series([], dtype=np.dtype(bool)),
-            "iso_time": pandas.Series([], dtype="datetime64[ns]"),
-        }
-        expected_result = DataFrame(
-            empty_columns, columns=["name", "number", "is_hurricane", "iso_time"],
-        )
-        tm.assert_frame_equal(df, expected_result, check_index_type=False)
-
     def test_one_row_one_column(self, project_id):
         df = gbq.read_gbq(
             "SELECT 3 as v",
@@ -644,7 +312,7 @@ class TestReadGBQIntegration(object):
             credentials=self.credentials,
             dialect="standard",
         )
-        expected_result = DataFrame(dict(v=[3]))
+        expected_result = DataFrame(dict(v=[3]), dtype="Int64")
         tm.assert_frame_equal(df, expected_result)
 
     def test_legacy_sql(self, project_id):
@@ -734,7 +402,7 @@ class TestReadGBQIntegration(object):
             configuration=config,
             dialect="legacy",
         )
-        tm.assert_frame_equal(df, DataFrame({"valid_result": [3]}))
+        tm.assert_frame_equal(df, DataFrame({"valid_result": [3]}, dtype="Int64"))
 
     def test_query_inside_configuration(self, project_id):
         query_no_use = 'SELECT "PI_WRONG" AS valid_string'
@@ -832,21 +500,6 @@ class TestReadGBQIntegration(object):
                     configuration=config,
                 )
 
-    def test_query_response_bytes(self):
-        assert self.gbq_connector.sizeof_fmt(999) == "999.0 B"
-        assert self.gbq_connector.sizeof_fmt(1024) == "1.0 KB"
-        assert self.gbq_connector.sizeof_fmt(1099) == "1.1 KB"
-        assert self.gbq_connector.sizeof_fmt(1044480) == "1020.0 KB"
-        assert self.gbq_connector.sizeof_fmt(1048576) == "1.0 MB"
-        assert self.gbq_connector.sizeof_fmt(1048576000) == "1000.0 MB"
-        assert self.gbq_connector.sizeof_fmt(1073741824) == "1.0 GB"
-        assert self.gbq_connector.sizeof_fmt(1.099512e12) == "1.0 TB"
-        assert self.gbq_connector.sizeof_fmt(1.125900e15) == "1.0 PB"
-        assert self.gbq_connector.sizeof_fmt(1.152922e18) == "1.0 EB"
-        assert self.gbq_connector.sizeof_fmt(1.180592e21) == "1.0 ZB"
-        assert self.gbq_connector.sizeof_fmt(1.208926e24) == "1.0 YB"
-        assert self.gbq_connector.sizeof_fmt(1.208926e28) == "10000.0 YB"
-
     def test_struct(self, project_id):
         query = """SELECT 1 int_field,
                    STRUCT("a" as letter, 1 as num) struct_field"""
@@ -857,7 +510,11 @@ class TestReadGBQIntegration(object):
             dialect="standard",
         )
         expected = DataFrame(
-            [[1, {"letter": "a", "num": 1}]], columns=["int_field", "struct_field"],
+            {
+                "int_field": pandas.Series([1], dtype="Int64"),
+                "struct_field": [{"letter": "a", "num": 1}],
+            },
+            columns=["int_field", "struct_field"],
         )
         tm.assert_frame_equal(df, expected)
 
@@ -889,7 +546,12 @@ class TestReadGBQIntegration(object):
             dialect="standard",
         )
         expected = DataFrame(
-            [["a", [""], 1], ["b", [], 0]], columns=["letter", "array_field", "len"],
+            {
+                "letter": ["a", "b"],
+                "array_field": [[""], []],
+                "len": pandas.Series([1, 0], dtype="Int64"),
+            },
+            columns=["letter", "array_field", "len"],
         )
         tm.assert_frame_equal(df, expected)
 
@@ -923,7 +585,13 @@ class TestReadGBQIntegration(object):
             credentials=self.credentials,
             dialect="standard",
         )
-        tm.assert_frame_equal(df, DataFrame([[[1.1, 2.2, 3.3], 4]], columns=["a", "b"]))
+        tm.assert_frame_equal(
+            df,
+            DataFrame(
+                {"a": [[1.1, 2.2, 3.3]], "b": pandas.Series([4], dtype="Int64")},
+                columns=["a", "b"],
+            ),
+        )
 
     def test_tokyo(self, tokyo_dataset, tokyo_table, project_id):
         df = gbq.read_gbq(
@@ -1036,7 +704,7 @@ class TestToGBQIntegration(object):
         test_id = "3"
         test_size = 10
         df = make_mixed_dataframe_v2(test_size)
-        df_different_schema = tm.makeMixedDataFrame()
+        df_different_schema = make_mixed_dataframe_v1()
 
         # Initialize table with sample data
         gbq.to_gbq(
@@ -1116,7 +784,7 @@ class TestToGBQIntegration(object):
         test_id = "4"
         test_size = 10
         df = make_mixed_dataframe_v2(test_size)
-        df_different_schema = tm.makeMixedDataFrame()
+        df_different_schema = make_mixed_dataframe_v1()
 
         # Initialize table with sample data
         gbq.to_gbq(
@@ -1240,7 +908,7 @@ class TestToGBQIntegration(object):
         result = result_df["s"].sort_values()
         expected = df["s"].sort_values()
 
-        tm.assert_numpy_array_equal(expected.values, result.values)
+        tm.assert_series_equal(expected, result)
 
     def test_upload_data_flexible_column_order(self, project_id):
         test_id = "13"
@@ -1269,7 +937,7 @@ class TestToGBQIntegration(object):
     def test_upload_data_with_valid_user_schema(self, project_id):
         # Issue #46; tests test scenarios with user-provided
         # schemas
-        df = tm.makeMixedDataFrame()
+        df = make_mixed_dataframe_v1()
         test_id = "18"
         test_schema = [
             {"name": "A", "type": "FLOAT"},
@@ -1291,7 +959,7 @@ class TestToGBQIntegration(object):
         )
 
     def test_upload_data_with_invalid_user_schema_raises_error(self, project_id):
-        df = tm.makeMixedDataFrame()
+        df = make_mixed_dataframe_v1()
         test_id = "19"
         test_schema = [
             {"name": "A", "type": "FLOAT"},
@@ -1310,7 +978,7 @@ class TestToGBQIntegration(object):
             )
 
     def test_upload_data_with_missing_schema_fields_raises_error(self, project_id):
-        df = tm.makeMixedDataFrame()
+        df = make_mixed_dataframe_v1()
         test_id = "20"
         test_schema = [
             {"name": "A", "type": "FLOAT"},
@@ -1366,7 +1034,7 @@ class TestToGBQIntegration(object):
         tm.assert_series_equal(expected, result)
 
     def test_upload_data_with_different_df_and_user_schema(self, project_id):
-        df = tm.makeMixedDataFrame()
+        df = make_mixed_dataframe_v1()
         df["A"] = df["A"].astype(str)
         df["B"] = df["B"].astype(str)
         test_id = "22"
@@ -1475,13 +1143,13 @@ def test_dataset_does_not_exist(gbq_dataset, random_dataset_id):
 
 
 def test_create_table(gbq_table):
-    schema = gbq._generate_bq_schema(tm.makeMixedDataFrame())
+    schema = gbq._generate_bq_schema(make_mixed_dataframe_v1())
     gbq_table.create("test_create_table", schema)
     assert gbq_table.exists("test_create_table")
 
 
 def test_create_table_already_exists(gbq_table):
-    schema = gbq._generate_bq_schema(tm.makeMixedDataFrame())
+    schema = gbq._generate_bq_schema(make_mixed_dataframe_v1())
     gbq_table.create("test_create_table_exists", schema)
     with pytest.raises(gbq.TableCreationError):
         gbq_table.create("test_create_table_exists", schema)
@@ -1505,11 +1173,6 @@ def test_delete_table(gbq_table):
     assert not gbq_table.exists("test_delete_table")
 
 
-def test_delete_table_not_found(gbq_table):
-    with pytest.raises(gbq.NotFoundException):
-        gbq_table.delete("test_delete_table_not_found")
-
-
 def test_create_table_data_dataset_does_not_exist(
     project, credentials, gbq_dataset, random_dataset_id
 ):
