@@ -8,6 +8,7 @@ import copy
 import datetime
 from unittest import mock
 
+import google.api_core.exceptions
 import numpy
 import pandas
 from pandas import DataFrame
@@ -31,45 +32,57 @@ def mock_get_credentials_no_project(*args, **kwargs):
     return mock_credentials, None
 
 
-def mock_get_credentials(*args, **kwargs):
-    import google.auth.credentials
-
-    mock_credentials = mock.create_autospec(google.auth.credentials.Credentials)
-    return mock_credentials, "default-project"
-
-
 @pytest.fixture
-def mock_service_account_credentials():
-    import google.oauth2.service_account
-
-    mock_credentials = mock.create_autospec(google.oauth2.service_account.Credentials)
-    return mock_credentials
-
-
-@pytest.fixture
-def mock_compute_engine_credentials():
-    import google.auth.compute_engine
-
-    mock_credentials = mock.create_autospec(google.auth.compute_engine.Credentials)
-    return mock_credentials
+def mock_query_job():
+    mock_query = mock.create_autospec(google.cloud.bigquery.QueryJob)
+    mock_query.job_id = "some-random-id"
+    mock_query.state = "DONE"
+    return mock_query
 
 
 @pytest.fixture(autouse=True)
-def no_auth(monkeypatch):
-    import pydata_google_auth
+def default_bigquery_client(mock_bigquery_client, mock_query_job):
+    mock_rows = mock.create_autospec(google.cloud.bigquery.table.RowIterator)
+    mock_rows.total_rows = 1
 
-    monkeypatch.setattr(pydata_google_auth, "default", mock_get_credentials)
+    mock_rows.__iter__.return_value = [(1,)]
+    mock_query_job.result.return_value = mock_rows
+    mock_bigquery_client.list_rows.return_value = mock_rows
+    mock_bigquery_client.query.return_value = mock_query_job
+
+    # Mock out SELECT 1 query results.
+    def generate_schema():
+        query = (
+            mock_bigquery_client.query.call_args[0][0]
+            if mock_bigquery_client.query.call_args
+            else ""
+        )
+        if query == "SELECT 1 AS int_col":
+            return [google.cloud.bigquery.SchemaField("int_col", "INTEGER")]
+        else:
+            return [google.cloud.bigquery.SchemaField("_f0", "INTEGER")]
+
+    type(mock_rows).schema = mock.PropertyMock(side_effect=generate_schema)
+
+    # Mock out get_table.
+    def get_table(table_ref_or_id, **kwargs):
+        return google.cloud.bigquery.Table(table_ref_or_id)
+
+    mock_bigquery_client.get_table.side_effect = get_table
+
+    return mock_bigquery_client
 
 
 @pytest.mark.parametrize(
     ("type_", "expected"),
     [
-        ("INTEGER", None),  # Can't handle NULL
-        ("BOOLEAN", None),  # Can't handle NULL
+        ("SOME_NEW_UNKNOWN_TYPE", None),
+        ("INTEGER", "Int64"),
         ("FLOAT", numpy.dtype(float)),
-        # TIMESTAMP will be localized after DataFrame construction.
-        ("TIMESTAMP", "datetime64[ns]"),
-        ("DATETIME", "datetime64[ns]"),
+        # TIMESTAMP/DATETIME might require object dtype for out-of-bounds
+        # values.
+        ("TIMESTAMP", None),
+        ("DATETIME", None),
     ],
 )
 def test__bqschema_to_nullsafe_dtypes(type_, expected):
@@ -82,25 +95,27 @@ def test__bqschema_to_nullsafe_dtypes(type_, expected):
         assert result == {"x": expected}
 
 
-def test_GbqConnector_get_client_w_old_bq(monkeypatch, mock_bigquery_client):
-    gbq._test_google_api_imports()
-    connector = _make_connector()
-    monkeypatch.setattr(
-        type(FEATURES),
-        "bigquery_has_client_info",
-        mock.PropertyMock(return_value=False),
-    )
-
-    connector.get_client()
-
-    # No client_info argument.
-    mock_bigquery_client.assert_called_with(credentials=mock.ANY, project=mock.ANY)
+@pytest.mark.parametrize(
+    ["query_or_table", "expected"],
+    [
+        ("SELECT 1", True),
+        ("SELECT\n1", True),
+        ("SELECT\t1", True),
+        ("dataset.table", False),
+        (" dataset.table ", False),
+        ("\r\ndataset.table\r\n", False),
+        ("project-id.dataset.table", False),
+        (" project-id.dataset.table ", False),
+        ("\r\nproject-id.dataset.table\r\n", False),
+    ],
+)
+def test__is_query(query_or_table, expected):
+    result = gbq._is_query(query_or_table)
+    assert result == expected
 
 
 def test_GbqConnector_get_client_w_new_bq(mock_bigquery_client):
     gbq._test_google_api_imports()
-    if not FEATURES.bigquery_has_client_info:
-        pytest.skip("google-cloud-bigquery missing client_info feature")
     pytest.importorskip("google.api_core.client_info")
 
     connector = _make_connector()
@@ -270,7 +285,7 @@ def test_to_gbq_w_project_table(mock_bigquery_client):
     assert table.project == "project_table"
 
 
-def test_to_gbq_creates_dataset(mock_bigquery_client):
+def test_to_gbq_create_dataset(mock_bigquery_client):
     import google.api_core.exceptions
 
     mock_bigquery_client.get_table.side_effect = google.api_core.exceptions.NotFound(
@@ -283,6 +298,111 @@ def test_to_gbq_creates_dataset(mock_bigquery_client):
     mock_bigquery_client.create_dataset.assert_called_with(mock.ANY)
 
 
+def test_dataset_create_already_exists_translates_exception(mock_bigquery_client):
+    connector = gbq._Dataset("my-project")
+    connector.client = mock_bigquery_client
+    mock_bigquery_client.get_dataset.return_value = object()
+    with pytest.raises(gbq.DatasetCreationError):
+        connector.create("already_exists")
+
+
+def test_dataset_exists_false(mock_bigquery_client):
+    connector = gbq._Dataset("my-project")
+    connector.client = mock_bigquery_client
+    mock_bigquery_client.get_dataset.side_effect = google.api_core.exceptions.NotFound(
+        "nope"
+    )
+    assert not connector.exists("not_exists")
+
+
+def test_dataset_exists_true(mock_bigquery_client):
+    connector = gbq._Dataset("my-project")
+    connector.client = mock_bigquery_client
+    mock_bigquery_client.get_dataset.return_value = object()
+    assert connector.exists("yes_exists")
+
+
+def test_dataset_exists_translates_exception(mock_bigquery_client):
+    connector = gbq._Dataset("my-project")
+    connector.client = mock_bigquery_client
+    mock_bigquery_client.get_dataset.side_effect = google.api_core.exceptions.InternalServerError(
+        "something went wrong"
+    )
+    with pytest.raises(gbq.GenericGBQException):
+        connector.exists("not_gonna_work")
+
+
+def test_table_create_already_exists(mock_bigquery_client):
+    connector = gbq._Table("my-project", "my_dataset")
+    connector.client = mock_bigquery_client
+    mock_bigquery_client.get_table.return_value = object()
+    with pytest.raises(gbq.TableCreationError):
+        connector.create(
+            "already_exists", {"fields": [{"name": "f", "type": "STRING"}]}
+        )
+
+
+def test_table_create_translates_exception(mock_bigquery_client):
+    connector = gbq._Table("my-project", "my_dataset")
+    connector.client = mock_bigquery_client
+    mock_bigquery_client.get_table.side_effect = google.api_core.exceptions.NotFound(
+        "nope"
+    )
+    mock_bigquery_client.create_table.side_effect = google.api_core.exceptions.InternalServerError(
+        "something went wrong"
+    )
+    with pytest.raises(gbq.GenericGBQException):
+        connector.create(
+            "not_gonna_work", {"fields": [{"name": "f", "type": "STRING"}]}
+        )
+
+
+def test_table_delete_notfound_ok(mock_bigquery_client):
+    connector = gbq._Table("my-project", "my_dataset")
+    connector.client = mock_bigquery_client
+    mock_bigquery_client.delete_table.side_effect = google.api_core.exceptions.NotFound(
+        "nope"
+    )
+    connector.delete("not_exists")
+    mock_bigquery_client.delete_table.assert_called_once()
+
+
+def test_table_delete_translates_exception(mock_bigquery_client):
+    connector = gbq._Table("my-project", "my_dataset")
+    connector.client = mock_bigquery_client
+    mock_bigquery_client.delete_table.side_effect = google.api_core.exceptions.InternalServerError(
+        "something went wrong"
+    )
+    with pytest.raises(gbq.GenericGBQException):
+        connector.delete("not_gonna_work")
+
+
+def test_table_exists_false(mock_bigquery_client):
+    connector = gbq._Table("my-project", "my_dataset")
+    connector.client = mock_bigquery_client
+    mock_bigquery_client.get_table.side_effect = google.api_core.exceptions.NotFound(
+        "nope"
+    )
+    assert not connector.exists("not_exists")
+
+
+def test_table_exists_true(mock_bigquery_client):
+    connector = gbq._Table("my-project", "my_dataset")
+    connector.client = mock_bigquery_client
+    mock_bigquery_client.get_table.return_value = object()
+    assert connector.exists("yes_exists")
+
+
+def test_table_exists_translates_exception(mock_bigquery_client):
+    connector = gbq._Table("my-project", "my_dataset")
+    connector.client = mock_bigquery_client
+    mock_bigquery_client.get_table.side_effect = google.api_core.exceptions.InternalServerError(
+        "something went wrong"
+    )
+    with pytest.raises(gbq.GenericGBQException):
+        connector.exists("not_gonna_work")
+
+
 def test_read_gbq_with_no_project_id_given_should_fail(monkeypatch):
     import pydata_google_auth
 
@@ -292,9 +412,10 @@ def test_read_gbq_with_no_project_id_given_should_fail(monkeypatch):
         gbq.read_gbq("SELECT 1", dialect="standard")
 
 
-def test_read_gbq_with_inferred_project_id(monkeypatch):
+def test_read_gbq_with_inferred_project_id(mock_bigquery_client):
     df = gbq.read_gbq("SELECT 1", dialect="standard")
     assert df is not None
+    mock_bigquery_client.query.assert_called_once()
 
 
 def test_read_gbq_with_inferred_project_id_from_service_account_credentials(
@@ -473,9 +594,6 @@ def test_read_gbq_passes_dtypes(mock_bigquery_client, mock_service_account_crede
 def test_read_gbq_use_bqstorage_api(
     mock_bigquery_client, mock_service_account_credentials
 ):
-    if not FEATURES.bigquery_has_bqstorage:
-        pytest.skip("requires BigQuery Storage API")
-
     mock_service_account_credentials.project_id = "service_account_project_id"
     df = gbq.read_gbq(
         "SELECT 1 AS int_col",
@@ -486,9 +604,17 @@ def test_read_gbq_use_bqstorage_api(
     assert df is not None
 
     mock_list_rows = mock_bigquery_client.list_rows("dest", max_results=100)
-    mock_list_rows.to_dataframe.assert_called_once_with(
-        create_bqstorage_client=True, dtypes=mock.ANY, progress_bar_type=mock.ANY,
-    )
+    if FEATURES.bigquery_needs_date_as_object:
+        mock_list_rows.to_dataframe.assert_called_once_with(
+            create_bqstorage_client=True,
+            dtypes=mock.ANY,
+            progress_bar_type=mock.ANY,
+            date_as_object=True,
+        )
+    else:
+        mock_list_rows.to_dataframe.assert_called_once_with(
+            create_bqstorage_client=True, dtypes=mock.ANY, progress_bar_type=mock.ANY,
+        )
 
 
 def test_read_gbq_calls_tqdm(mock_bigquery_client, mock_service_account_credentials):
@@ -505,3 +631,107 @@ def test_read_gbq_calls_tqdm(mock_bigquery_client, mock_service_account_credenti
 
     _, to_dataframe_kwargs = mock_list_rows.to_dataframe.call_args
     assert to_dataframe_kwargs["progress_bar_type"] == "foobar"
+
+
+def test_read_gbq_with_full_table_id(
+    mock_bigquery_client, mock_service_account_credentials
+):
+    mock_service_account_credentials.project_id = "service_account_project_id"
+    df = gbq.read_gbq(
+        "my-project.my_dataset.read_gbq_table",
+        credentials=mock_service_account_credentials,
+        project_id="param-project",
+    )
+    assert df is not None
+
+    mock_bigquery_client.query.assert_not_called()
+    sent_table = mock_bigquery_client.list_rows.call_args[0][0]
+    assert sent_table.project == "my-project"
+    assert sent_table.dataset_id == "my_dataset"
+    assert sent_table.table_id == "read_gbq_table"
+
+
+def test_read_gbq_with_partial_table_id(
+    mock_bigquery_client, mock_service_account_credentials
+):
+    mock_service_account_credentials.project_id = "service_account_project_id"
+    df = gbq.read_gbq(
+        "my_dataset.read_gbq_table",
+        credentials=mock_service_account_credentials,
+        project_id="param-project",
+    )
+    assert df is not None
+
+    mock_bigquery_client.query.assert_not_called()
+    sent_table = mock_bigquery_client.list_rows.call_args[0][0]
+    assert sent_table.project == "param-project"
+    assert sent_table.dataset_id == "my_dataset"
+    assert sent_table.table_id == "read_gbq_table"
+
+
+def test_read_gbq_bypasses_query_with_table_id_and_max_results(
+    mock_bigquery_client, mock_service_account_credentials
+):
+    mock_service_account_credentials.project_id = "service_account_project_id"
+    df = gbq.read_gbq(
+        "my-project.my_dataset.read_gbq_table",
+        credentials=mock_service_account_credentials,
+        max_results=11,
+    )
+    assert df is not None
+
+    mock_bigquery_client.query.assert_not_called()
+    sent_table = mock_bigquery_client.list_rows.call_args[0][0]
+    assert sent_table.project == "my-project"
+    assert sent_table.dataset_id == "my_dataset"
+    assert sent_table.table_id == "read_gbq_table"
+    sent_max_results = mock_bigquery_client.list_rows.call_args[1]["max_results"]
+    assert sent_max_results == 11
+
+
+def test_read_gbq_with_list_rows_error_translates_exception(
+    mock_bigquery_client, mock_service_account_credentials
+):
+    mock_bigquery_client.list_rows.side_effect = (
+        google.api_core.exceptions.NotFound("table not found"),
+    )
+
+    with pytest.raises(gbq.GenericGBQException, match="table not found"):
+        gbq.read_gbq(
+            "my-project.my_dataset.read_gbq_table",
+            credentials=mock_service_account_credentials,
+        )
+
+
+@pytest.mark.parametrize(
+    ["size_in_bytes", "formatted_text"],
+    [
+        (999, "999.0 B"),
+        (1024, "1.0 KB"),
+        (1099, "1.1 KB"),
+        (1044480, "1020.0 KB"),
+        (1048576, "1.0 MB"),
+        (1048576000, "1000.0 MB"),
+        (1073741824, "1.0 GB"),
+        (1.099512e12, "1.0 TB"),
+        (1.125900e15, "1.0 PB"),
+        (1.152922e18, "1.0 EB"),
+        (1.180592e21, "1.0 ZB"),
+        (1.208926e24, "1.0 YB"),
+        (1.208926e28, "10000.0 YB"),
+    ],
+)
+def test_query_response_bytes(size_in_bytes, formatted_text):
+    assert gbq.GbqConnector.sizeof_fmt(size_in_bytes) == formatted_text
+
+
+def test_run_query_with_dml_query(mock_bigquery_client, mock_query_job):
+    """
+    Don't attempt to download results from a DML query / query with no results.
+
+    https://github.com/googleapis/python-bigquery-pandas/issues/481
+    """
+    connector = _make_connector()
+    type(mock_query_job).destination = mock.PropertyMock(return_value=None)
+    connector.run_query("UPDATE tablename SET value = '';")
+    mock_bigquery_client.list_rows.assert_not_called()
